-- ========================================================
-- ========         coloData Lib test,         ============
-- ========         Pakoskyfrog 2013/03/08     ============
-- ========================================================

------------------------
--  Description
--[[
    This is a test file for the library colorData.
    colorData is designed to manipulate colors and provide some handy shortcuts.
]]

------------------------
--  Dependencies
cd = require 'colorData'

------------------------
--  Init

local menu = {}
menu.cmds = {
    -- {'text to display','key to activate'}
    {'Set display to named colors','n'},
    {'Set display to indexed color lists','i'},
    {'Set display to gradient color lists','g'},
    {'Display nothing', 'space'},
    {'Close','escape'}
}
menu.fonts = {
    big    = love.graphics.newFont(40),
    normal = love.graphics.newFont(18),
    small  = love.graphics.newFont(12),
    mini   = love.graphics.newFont(8)
}
local currentName  = 'black'
local currentColor = {0,0,0}

function dummy()
    -- nothing here : dummy function
end

------------------------
--  LOVE callbacks
function drawMenu()
    love.graphics.setFont(menu.fonts.big)
    love.graphics.setColor(255,255,255)
    love.graphics.print("TESTS OF colorData Lib :", 5, 5)
    
    love.graphics.setFont(menu.fonts.normal)
    love.graphics.setColor(200,200,200)
    local y = 50
    for index, cmd in ipairs(menu.cmds) do
        love.graphics.print('['..cmd[2]..'] '..cmd[1], 15, y)
        y = y + 20
    end
    
    love.graphics.setFont(menu.fonts.mini)
    love.graphics.setColor(0,63,0)
    love.graphics.print("Pakoskyfrog 2013", 720, 582)
end

drawColor = dummy
function love.draw()
    -- draw menu
    drawMenu()
    
    -- draw current func color
    drawColor(100,200)
end

keyb = dummy
function love.keypressed(key)
    -- functional state machine

    if key=='escape' then
        love.event.push('quit')
        
    elseif key=='n' then
        drawColor = drawNamed
        keyb      = keyNamed
        mouse     = mouseNamed
        currentColor, currentName = goIterNamed(0)

    elseif key == 'g' then
        drawColor = drawGradient
        keyb      = keyGradient
        mouse     = mouseGradient
        currentColor, currentName = goIterGradients(0)
    
    elseif key == 'i' then
        drawColor = drawIndexed
        keyb      = keyIndexed
        mouse     = mouseIndexed
        currentColor, currentName = goIterIndexed(0)
    
    elseif key == ' ' then
        -- reinit
        drawColor = dummy
        keyb      = dummy
        mouse     = dummy
        
    end
    keyb(key)
end

mouse = dummy
function love.mousepressed(x,y, btn)
    mouse(x,y,btn)
end


------------------------
--  functions

local function goIterNamedCreate()
    --------------------
    --  Countor creator
    local index = 1
    return function (sens)
        if (index == #cd.listOfColors and sens>0) then
            index = 0
        end
        if (index == 1 and sens<0) then
            index = #cd.listOfColors+1
        end
        index = index + sens
        return cd.getNamedColor(cd.listOfColors[index]), cd.listOfColors[index]
    end
end
goIterNamed = goIterNamedCreate()

function drawNamed(ox,oy)
    love.graphics.setFont(menu.fonts.normal)
    love.graphics.setColor(255,255,255)
    love.graphics.print(currentName, ox, oy)
    
    love.graphics.setLineWidth(6)
    
    love.graphics.setColor(127,127,127)
    love.graphics.rectangle('fill', ox+20-6, oy+30-6, 312, 75*3/2+12)
    
    love.graphics.setColor(currentColor)
    love.graphics.rectangle('fill', ox+20, oy+30, 200, 75)
    love.graphics.setColor(255,255,255)
    love.graphics.rectangle('line', ox+20, oy+30, 200, 75)
    
    love.graphics.setColor(currentColor)
    love.graphics.rectangle('fill', ox+120, oy+30+75/2, 200, 75)
    love.graphics.setColor(0,0,0)
    love.graphics.rectangle('line', ox+120, oy+30+75/2, 200, 75)
end

function keyNamed(key)
    if key == 'left' or key == 'up' then
        currentColor, currentName = goIterNamed(-1)
    elseif key == 'right' or key == 'down' then
        currentColor, currentName = goIterNamed(1)
    end
    
end
function mouseNamed(x,y,btn)
    if btn == 'wu' or btn == 'l' then
        currentColor, currentName = goIterNamed(1)
    elseif btn == 'wd' or btn == 'r' then
        currentColor, currentName = goIterNamed(-1)
    end
    
end

-----------------------------

local function goIterGradientsCreate()
    --------------------
    --  Countor creator
    local index = 1
    return function (sens)
        if (index == #cd.listOfGradients and sens>0) then
            index = 0
        end
        if (index == 1 and sens<0) then
            index = #cd.listOfGradients+1
        end
        index = index + sens
        return cd.getGradientColors(cd.listOfGradients[index]), cd.listOfGradients[index]
    end
end
goIterGradients = goIterGradientsCreate()

function drawGradient(ox,oy)
    love.graphics.setFont(menu.fonts.normal)
    love.graphics.setColor(255,255,255)
    love.graphics.print(currentName, ox, oy)
    
    love.graphics.setLineWidth(6)
    
    local w = love.graphics.getWidth()-2*ox
    local di = 1/(w-1)
    
    love.graphics.setColor(127,127,127)
    love.graphics.rectangle('line', ox+20-3, oy+30-3, w+6, 75+6)
    
    local grad = currentColor
    
    for i = 0, w-1 do
        love.graphics.setColor(cd.evalGradientAt(grad, i*di))
        love.graphics.line(ox+i+20,oy+30, ox+i+20,oy+75+30)
    end
end

function keyGradient(key)
    if key == 'left' or key == 'up' then
        currentColor, currentName = goIterGradients(-1)
    elseif key == 'right' or key == 'down' then
        currentColor, currentName = goIterGradients(1)
    end
    
end

function mouseGradient(x,y,btn)
    if btn == 'wu' or btn == 'l' then
        currentColor, currentName = goIterGradients(1)
    elseif btn == 'wd' or btn == 'r' then
        currentColor, currentName = goIterGradients(-1)
    end
    
end

-----------------------------

local function goIterIndexedCreate()
    --------------------
    --  Countor creator
    local index = 1
    return function (sens)
        if (index == #cd.listOfIndexed and sens>0) then
            index = 0
        end
        if (index == 1 and sens<0) then
            index = #cd.listOfIndexed+1
        end
        index = index + sens
        return cd.getIndexedColors(cd.listOfIndexed[index]), cd.listOfIndexed[index]
    end
end
goIterIndexed = goIterIndexedCreate()

function keyIndexed(key)
    if key == 'left' or key == 'up' then
        currentColor, currentName = goIterIndexed(-1)
    elseif key == 'right' or key == 'down' then
        currentColor, currentName = goIterIndexed(1)
    end
    
end

function mouseIndexed(x,y,btn)
    if btn == 'wu' or btn == 'l' then
        currentColor, currentName = goIterIndexed(1)
    elseif btn == 'wd' or btn == 'r' then
        currentColor, currentName = goIterIndexed(-1)
    end
    
end

function drawIndexed(ox,oy)
    love.graphics.setFont(menu.fonts.normal)
    love.graphics.setColor(255,255,255)
    love.graphics.print(currentName, ox, oy)
    
    love.graphics.setLineWidth(6)
    
    local w = love.graphics.getWidth()-2*ox
    local N = #currentColor
    local dw = 10
    local dn = (w)/(N) - dw
    
    love.graphics.setColor(127,127,127)
    love.graphics.rectangle('fill', ox+20-6, oy+30-6, w+2, 75+6)
    
    for r = 0, N-1 do
        love.graphics.setColor(currentColor[r+1])
        love.graphics.rectangle('fill', ox+(dn+dw)*r+20-3, oy+30-3, dn+6, 69+6)
    end
    
    local ypos = oy+150
    local dy = 1+menu.fonts.small:getHeight()
    love.graphics.setFont(menu.fonts.small)
    for index, color in ipairs(currentColor) do
        love.graphics.setColor(color)
        love.graphics.print(color[1]..":"..color[2]..":"..color[3], ox, ypos+(index-3)*dy)
    end
    
end








